#include "vidsch_dfs_e3k.h"
#include "vidsch.h"
#include "vidschi.h"
#include "chip_include_e3k.h"
#include "vidsch_engine_e3k.h"
#include "register_e3k.h"

void vidschi_init_dvfs_setting_004(adapter_t *adapter)
{
    vidsch_mgr_t *sch_mgr = NULL;
    vidschedule_t *schedule = adapter->schedule;

    Reg_Reg_Dvfs_Cfg_En         reg_dvfs_cfg_en;
    Reg_Reg_Dvfs_Cfg_Misc0      reg_dvfs_cfg_misc0;

    Reg_Reg_3d_Dvfs_Checksize   reg_3d_dvfs_checksize   = {.uint = 0x2000};
    Reg_Reg_3d_Dvfs_H_Th        reg_3d_dvfs_h_threshold = {.uint = 0x1500};
    Reg_Reg_3d_Dvfs_L_Th        reg_3d_dvfs_l_threshold = {.uint = 0x0500};
    Reg_Reg_3d_Dvfs_H_Th_Mem    reg_3d_dvfs_h_th_mem    = {.uint = 0x1800};

    Reg_Reg_Vcp0_Dvfs_Checksize reg_vcp0_dvfs_checksize = {.uint = 0x2000};
    Reg_Reg_Vcp0_Dvfs_H_Th      reg_vcp0_dvfs_h_th      = {.uint = 0x1500};
    Reg_Reg_Vcp0_Dvfs_L_Th      reg_vcp0_dvfs_l_th      = {.uint = 0x0500};
    Reg_Reg_Vcp0_Dvfs_H_Th_Mem  reg_vcp0_dvfs_h_th_mem  = {.uint = 0x1800};

    Reg_Reg_Vcp1_Dvfs_Checksize reg_vcp1_dvfs_checksize = {.uint = 0x2000};
    Reg_Reg_Vcp1_Dvfs_H_Th      reg_vcp1_dvfs_h_th      = {.uint = 0x1500};
    Reg_Reg_Vcp1_Dvfs_L_Th      reg_vcp1_dvfs_l_th      = {.uint = 0x0500};
    Reg_Reg_Vcp1_Dvfs_H_Th_Mem  reg_vcp1_dvfs_h_th_mem  = {.uint = 0x1800};

    Reg_Reg_Vpp_Dvfs_Checksize reg_vpp_dvfs_checksize = {.uint = 0x2000};
    Reg_Reg_Vpp_Dvfs_H_Th      reg_vpp_dvfs_h_th      = {.uint = 0x1500};
    Reg_Reg_Vpp_Dvfs_L_Th      reg_vpp_dvfs_l_th      = {.uint = 0x0500};
    Reg_Reg_Vpp_Dvfs_H_Th_Mem  reg_vpp_dvfs_h_th_mem  = {.uint = 0x1800};

    //Before ECLK change, CRAB_C must set to 0, otherwise snoop request will make BIU hang
    zx_write8(adapter->mmio + 0x8AAB, 0);

    reg_dvfs_cfg_en.uint = 0;
    reg_dvfs_cfg_en.reg.Dvfs_3d_Auto_En   = adapter->pm_caps.gfx_dvfs_auto;
    reg_dvfs_cfg_en.reg.Dvfs_Vcp0_Auto_En = adapter->pm_caps.vcp_dvfs_auto;
    reg_dvfs_cfg_en.reg.Dvfs_Vcp1_Auto_En = adapter->pm_caps.vcp_dvfs_auto;
    reg_dvfs_cfg_en.reg.Dvfs_Vpp_Auto_En  = adapter->pm_caps.vpp_dvfs_auto;

    if(adapter->pm_caps.gfx_mem_dvfs || adapter->pm_caps.vcp_mem_dvfs || adapter->pm_caps.vpp_mem_dvfs)
    {
        reg_dvfs_cfg_en.reg.Reg_Memory_Dvfs_En = 1;

        reg_dvfs_cfg_misc0.reg.Dvfs_3d_M2m_Dvfs_En   = adapter->pm_caps.gfx_mem_dvfs;
        reg_dvfs_cfg_misc0.reg.Dvfs_Vcp0_M2m_Dvfs_En = adapter->pm_caps.vcp_mem_dvfs;
        reg_dvfs_cfg_misc0.reg.Dvfs_Vcp1_M2m_Dvfs_En = adapter->pm_caps.vcp_mem_dvfs;
        reg_dvfs_cfg_misc0.reg.Dvfs_Vpp_M2m_Dvfs_En  = adapter->pm_caps.vpp_mem_dvfs;
        /*******************miu to engine ratio***************
        *000:engine<16/16 miu
        *001:engine<15/16 miu
        *010:engine<14/16 miu
        *011:engine<13/16 miu
        *100:engine<12/16 miu
        *101:engine<11/16 miu
        *110:engine<10/16 miu
        *111:engine< 9/16 miu
        ******************************************************/
        reg_dvfs_cfg_misc0.reg.Dvfs_3d_M2e_Ratio     = 0x7;
        reg_dvfs_cfg_misc0.reg.Dvfs_Vcp0_M2e_Ratio   = 0x7;
        reg_dvfs_cfg_misc0.reg.Dvfs_Vcp1_M2e_Ratio   = 0x7;
        reg_dvfs_cfg_misc0.reg.Dvfs_Vpp_M2e_Ratio    = 0x7;
    }

    reg_dvfs_cfg_misc0.reg.Dvfs_3d_Output_Mode   = adapter->pm_caps.gfx_output_mode;
    reg_dvfs_cfg_misc0.reg.Dvfs_Vcp0_Output_Mode = adapter->pm_caps.vcp_output_mode;
    reg_dvfs_cfg_misc0.reg.Dvfs_Vcp1_Output_Mode = adapter->pm_caps.vcp_output_mode;
    reg_dvfs_cfg_misc0.reg.Dvfs_Vpp_Output_Mode  = adapter->pm_caps.vpp_output_mode;

    //3d_cnt_mode:
    //0--Max(EU,FFU,TU)
    //1--Max(EU,FFU,TU,TSG,L2)
    //2--OR(EU,FFU,TU)
    //3--OR(EU,FFU,TU,TSG,L2)
    reg_dvfs_cfg_misc0.reg.Dvfs_3d_Cnt_Mode = 3;

    //3d_cnt_mux: 0--ALU  1--Workload
    reg_dvfs_cfg_misc0.reg.Dvfs_3d_Cnt_Mux	= 0;

    zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Reg_3d_Dvfs_Checksize_Offset*4, reg_3d_dvfs_checksize.uint);
    zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Reg_3d_Dvfs_H_Th_Offset*4, reg_3d_dvfs_h_threshold.uint);
    zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Reg_3d_Dvfs_L_Th_Offset*4, reg_3d_dvfs_l_threshold.uint);
    zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Reg_3d_Dvfs_H_Th_Mem_Offset*4, reg_3d_dvfs_h_th_mem.uint);

    zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Reg_Vcp0_Dvfs_Checksize_Offset*4, reg_vcp0_dvfs_checksize.uint);
    zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Reg_Vcp0_Dvfs_H_Th_Offset*4, reg_vcp0_dvfs_h_th.uint);
    zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Reg_Vcp0_Dvfs_L_Th_Offset*4, reg_vcp0_dvfs_l_th.uint);
    zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Reg_Vcp0_Dvfs_H_Th_Mem_Offset*4, reg_vcp0_dvfs_h_th_mem.uint);

    zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Reg_Vcp1_Dvfs_Checksize_Offset*4, reg_vcp1_dvfs_checksize.uint);
    zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Reg_Vcp1_Dvfs_H_Th_Offset*4, reg_vcp1_dvfs_h_th.uint);
    zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Reg_Vcp1_Dvfs_L_Th_Offset*4, reg_vcp1_dvfs_l_th.uint);
    zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Reg_Vcp1_Dvfs_H_Th_Mem_Offset*4, reg_vcp1_dvfs_h_th_mem.uint);

    zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Reg_Vpp_Dvfs_Checksize_Offset*4, reg_vpp_dvfs_checksize.uint);
    zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Reg_Vpp_Dvfs_H_Th_Offset*4, reg_vpp_dvfs_h_th.uint);
    zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Reg_Vpp_Dvfs_L_Th_Offset*4, reg_vpp_dvfs_l_th.uint);
    zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Reg_Vpp_Dvfs_H_Th_Mem_Offset*4, reg_vpp_dvfs_h_th_mem.uint);

    zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Reg_Dvfs_Cfg_Misc0_Offset * 4, reg_dvfs_cfg_misc0.uint);
    zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Reg_Dvfs_Cfg_En_Offset * 4, reg_dvfs_cfg_en.uint);

    zx_info("init dvfs done.\n");
}

void vidsch_disable_dvfs_004(adapter_t *adapter)
{
    zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Reg_Dvfs_Cfg_En_Offset * 4, 0);
    zx_info("disable dvfs done.\n");
}

void vidsch_dfs_deinit_e3k(adapter_t *adapter)
{
    unsigned int reg_value = 0;

    //enable the bit of all reg protection. disable cpu R/W of dfs mmio.
    //reg_value = zx_read32(adapter->mmio +   );
    //reg_value |= (1 << 8);
    //zx_write32(adapter->mmio +   , reg_value);
}

void vidsch_dfs_tuning_e3k(adapter_t *adapter)
{
}

int vidsch_power_clock_on_off_e3k(vidsch_mgr_t *sch_mgr, unsigned int off)
{
    adapter_t * adapter = sch_mgr->adapter;
    vidschedule_t *schedule = adapter->schedule;
    unsigned long flags = 0;
    int delay_times = 2000 * 100; // 2000 * 100 * 1 us = 200ms
    int i;

    Reg_Pwr_Mgr_Sw_Cfg0 reg_sw_cfg0;
    Reg_Pwr_Mgr_Status1 reg_status1;

    reg_status1.uint = 0;
    flags = zx_spin_lock_irqsave(schedule->power_status_lock);
    //power_trace("[CG] used_gpc_mask_bit:%x\n", used_gpc_mask_bit);

    reg_sw_cfg0.uint = zx_read32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Pwr_Mgr_Sw_Cfg0_Offset * 4);
#define WRITE_SW_CFG_REG_GFX(type, val) \
        do \
        {  \
           reg_sw_cfg0.reg.Sw_Ctl_3d_Centroy_##type = val; \
           reg_sw_cfg0.reg.Sw_Ctl_3d_Slc0_##type = val;    \
           reg_sw_cfg0.reg.Sw_Ctl_3d_Slc1_##type = val;    \
           zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Pwr_Mgr_Sw_Cfg0_Offset * 4, reg_sw_cfg0.uint); \
           zx_udelay(1); \
        } while(0)

#define WRITE_SW_CFG0_REG_VIDEO(type, val)                                        \
    do {                                                                          \
         reg_sw_cfg0.reg.Sw_Ctl_##type = val;                                     \
         zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Pwr_Mgr_Sw_Cfg0_Offset * 4, reg_sw_cfg0.uint); \
         zx_udelay(1);                                                            \
    } while(0)

#define WRITE_SW_CFG0_REG_VIDEO_POWER_OFF(video_reg)                                            \
        do {                                                                                    \
            WRITE_SW_CFG0_REG_VIDEO(video_reg##_Clken, 0);                                      \
            WRITE_SW_CFG0_REG_VIDEO(video_reg##_Isoon, 1);                                      \
            WRITE_SW_CFG0_REG_VIDEO(video_reg##_Pwron, 0);                                      \
                                                                                                \
            reg_status1.uint = zx_read32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Pwr_Mgr_Status1_Offset * 4);       \
                                                                                                \
            while(reg_status1.reg.Csp_Pwm_##video_reg##_Pwron_Be != 0)                          \
            {                                                                                   \
                  delay_times--;                                                                \
                  if (!delay_times)                                                             \
                  {                                                                             \
                      zx_info("cannot power off video!!!\n");                                   \
                      break;                                                                    \
                  }                                                                             \
                  zx_udelay(1);                                                                 \
                  reg_status1.uint = zx_read32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Pwr_Mgr_Status1_Offset * 4); \
            }                                                                                   \
        }while (0)


#define WRITE_CFG0_REG_VIDEO_POWER_ON(video_reg) \
    do { \
        WRITE_SW_CFG0_REG_VIDEO(video_reg##_Pwron, 1);                                      \
        reg_status1.uint = zx_read32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Pwr_Mgr_Status1_Offset * 4);       \
        while(reg_status1.reg.Csp_Pwm_##video_reg##_Pwron_Be == 0)                          \
        {                                                                                   \
              delay_times--;                                                                \
              if (!delay_times)                                                             \
              {                                                                             \
                  zx_info("cannot power on video!!!\n");                                    \
                  break;                                                                    \
              }                                                                             \
              zx_udelay(1);                                                                 \
              reg_status1.uint = zx_read32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Pwr_Mgr_Status1_Offset * 4); \
         }                                                                                  \
         WRITE_SW_CFG0_REG_VIDEO(video_reg##_Rston, 0);                                     \
         WRITE_SW_CFG0_REG_VIDEO(video_reg##_Clken, 1);                                     \
         WRITE_SW_CFG0_REG_VIDEO(video_reg##_Rston, 1);                                     \
         WRITE_SW_CFG0_REG_VIDEO(video_reg##_Isoon, 0);                                     \
    }while (0)

    if(off)
    {
        switch(sch_mgr->engine_index)
        {
            case RB_INDEX_GFXL:
            case RB_INDEX_GFXH: 
                //write_reg_e3k(adapter->mmio, CR_C, 0x01, 0, 0x8F);
                if (adapter->pm_caps.gfx_pg_manual)
                {
                    WRITE_SW_CFG_REG_GFX(Clken, 0);
                    WRITE_SW_CFG_REG_GFX(Isoon, 1);
                    WRITE_SW_CFG_REG_GFX(Pwron, 0);

                    reg_status1.uint = zx_read32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Pwr_Mgr_Status1_Offset * 4);

                    while(reg_status1.reg.Csp_Pwm_Central_Pwron_Be != 0 ||
                          reg_status1.reg.Csp_Pwm_Slc0_Pwron_Be != 0 ||
                          reg_status1.reg.Csp_Pwm_Slc1_Pwron_Be != 0)
                    {
                          delay_times--;
                          if (!delay_times)
                          {
                              zx_info("cannot power off gfx!!!\n");
                              for (i = Reg_Pwr_Mgr_Cfg_En_Offset; i <= Reg_Pwr_Mgr_Status1_Offset; i++)
                              {
                                   zx_info("reg %x value: %x\n", i, zx_read32(adapter->mmio + MMIO_CSP_START_ADDRESS + i * 4));
                              }
                              break;
                          }
                          zx_udelay(1);
                          reg_status1.uint = zx_read32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Pwr_Mgr_Status1_Offset * 4);
                    }
                }
                break;
            case RB_INDEX_VCP0:
                //write_reg_e3k(adapter->mmio, CR_C, 0x01, 0, 0xF7);
                if (adapter->pm_caps.vcp_pg_manual)
                {
                    WRITE_SW_CFG0_REG_VIDEO_POWER_OFF(S3vd0);
                }
                break;
            case RB_INDEX_VCP1:
                //write_reg_e3k(adapter->mmio, CR_C, 0x01, 0, 0xFB);
                if (adapter->pm_caps.vcp_pg_manual)
                {
                    WRITE_SW_CFG0_REG_VIDEO_POWER_OFF(S3vd1);
                }
                break;
            case RB_INDEX_VPP:
                //write_reg_e3k(adapter->mmio, CR_C, 0x01, 0, 0xFD);
                if (adapter->pm_caps.vpp_pg_manual)
                {
                    WRITE_SW_CFG0_REG_VIDEO_POWER_OFF(Vpp);
                }
                break;
            default:
                  break;
        }

        power_trace("[CG] engine %d do cg power off,  CR01:%x\n", sch_mgr->engine_index, read_reg_e3k(adapter->mmio, CR_C, 0x01));
    }
    else
    {
        switch(sch_mgr->engine_index)
        {
            case RB_INDEX_GFXL:
            case RB_INDEX_GFXH:
                //write_reg_e3k(adapter->mmio, CR_C, 0x01, (0x70 & used_gpc_mask_bit), 0x8F);
                if (adapter->pm_caps.gfx_pg_manual)
                {
                    reg_sw_cfg0.reg.Sw_Ctl_3d_Centroy_Pwron = 1;
                    reg_sw_cfg0.reg.Sw_Ctl_3d_Slc0_Pwron = 1;
                    reg_sw_cfg0.reg.Sw_Ctl_3d_Slc1_Pwron = 1;

                    zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Pwr_Mgr_Sw_Cfg0_Offset * 4, reg_sw_cfg0.uint);
                    reg_status1.uint = zx_read32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Pwr_Mgr_Status1_Offset * 4);

                    while(reg_status1.reg.Csp_Pwm_Central_Pwron_Be == 0 ||
                          reg_status1.reg.Csp_Pwm_Slc0_Pwron_Be == 0 ||
                          reg_status1.reg.Csp_Pwm_Slc1_Pwron_Be == 0)
                    {
                          delay_times--;
                          if (!delay_times)
                          {
                              zx_info("cannot power on gfx!!!\n");
                              for (i = Reg_Pwr_Mgr_Cfg_En_Offset; i <= Reg_Pwr_Mgr_Status1_Offset; i++)
                              {
                                   zx_info("reg %x value: %x\n", i, zx_read32(adapter->mmio + MMIO_CSP_START_ADDRESS + i * 4));
                              }
                              break;
                          }
                          zx_udelay(1);
                          reg_status1.uint = zx_read32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Pwr_Mgr_Status1_Offset * 4);

                    }

                    WRITE_SW_CFG_REG_GFX(Rston, 0);
                    WRITE_SW_CFG_REG_GFX(Clken, 1);
                    WRITE_SW_CFG_REG_GFX(Rston, 1);
                    WRITE_SW_CFG_REG_GFX(Isoon, 0);

                    zx_udelay(1);
                }
                break;
            case RB_INDEX_VCP0:
                //write_reg_e3k(adapter->mmio, CR_C, 0x01, 8, 0xF7);
                if (adapter->pm_caps.vcp_pg_manual)
                {
                    WRITE_CFG0_REG_VIDEO_POWER_ON(S3vd0);
                }
                break;
            case RB_INDEX_VCP1:
                //write_reg_e3k(adapter->mmio, CR_C, 0x01, 4, 0xFB);
                if (adapter->pm_caps.vcp_pg_manual)
                {
                    WRITE_CFG0_REG_VIDEO_POWER_ON(S3vd1);
                }
                break;
            case RB_INDEX_VPP:
                //write_reg_e3k(adapter->mmio, CR_C, 0x01, 2, 0xFD);
                if (adapter->pm_caps.vpp_pg_manual)
                {
                    WRITE_CFG0_REG_VIDEO_POWER_ON(Vpp);
                }
                break;
            default:
                  break;
        }
        power_trace("[CG] engine %d do cg power on,  CR01:%x\n", sch_mgr->engine_index, read_reg_e3k(adapter->mmio, CR_C, 0x01));
    }

    zx_spin_unlock_irqrestore(schedule->power_status_lock, flags);

    return S_OK;
}

void vidsch_power_tuning_e3k(adapter_t *adapter)
{
    vidschedule_t *schedule = adapter->schedule;
    vidsch_mgr_t  *sch_mgr = NULL;
    int engine_index;

    unsigned int engine_pwm_auto_mode [] = {
    adapter->pm_caps.gfx_pg_auto, //RB_INDEX_GFXL
    adapter->pm_caps.gfx_pg_auto, //RB_INDEX_GFXH
    1, //RB_INDEX_CSH
    1, //RB_INDEX_CSL0
    1, //RB_INDEX_CSL1
    1, //RB_INDEX_CSL2
    1, //RB_INDEX_CSL3
    adapter->pm_caps.vcp_pg_auto, //RB_INDEX_VCP0
    adapter->pm_caps.vcp_pg_auto, //RB_INDEX_VCP1
    adapter->pm_caps.vpp_pg_auto, //RB_INDEX_VPP
    };

    if (!adapter->pwm_level.EnableClockGating || !adapter->pm_caps.pwm_manual)
    {
        return;
    }

    for (engine_index = adapter->active_engine_count-1; engine_index >= 0; engine_index--)
    {
        int ret = S_OK;
        unsigned long flags=0;
        sch_mgr = adapter->sch_mgr[engine_index];

        if (sch_mgr == NULL)  continue;

        if (engine_pwm_auto_mode[sch_mgr->engine_index]) continue;

        vidsch_update_engine_idle_status(adapter, (1 << engine_index));
        if (sch_mgr->completely_idle)
        {
            flags = zx_spin_lock_irqsave(sch_mgr->power_status_lock);
            if (sch_mgr->chip_func->power_clock &&
                sch_mgr->engine_dvfs_power_on &&
                vidsch_is_fence_back(sch_mgr->adapter, sch_mgr->engine_index, sch_mgr->last_send_fence_id))
            {
                sch_mgr->chip_func->power_clock(sch_mgr, TRUE);
                sch_mgr->engine_dvfs_power_on = FALSE;
            }
            zx_spin_unlock_irqrestore(sch_mgr->power_status_lock, flags);
        }
    }
}

void vidsch_pll_on_off_e3k(adapter_t* adapter, unsigned int pll_on)
{
     unsigned int epll_val, vpll_val;
     vidschedule_t* schedule = adapter->schedule;
     Reg_Reg_Dvfs_Cfg_En         reg_dvfs_cfg_en = {.uint = 0};

     if (schedule->pll_on == pll_on)
     {
         return;
     }

     if (pll_on)
     {
         // turn on epll
         epll_val = 0x1;
         zx_write32(adapter->mmio + 0x64018, epll_val); // EPLL_PU
         zx_udelay(20);
         epll_val |= 0x10;
         zx_write32(adapter->mmio + 0x64018, epll_val); // EPLL_RST
         zx_udelay(20);

         // turn on vpll
         vpll_val = 0x1;
         zx_write32(adapter->mmio + 0x64020, vpll_val);  // VPLL_PU
         zx_udelay(20);
         vpll_val |= 0x10;
         zx_write32(adapter->mmio + 0x64020, vpll_val); // VPLL_RST
         zx_udelay(20);

         // enable dvfs after turn on pll
         zx_write32(adapter->mmio + 0x68900, 1);
         zx_write32(adapter->mmio + 0x68904, 1);
         zx_write32(adapter->mmio + 0x68908, 1);
         zx_write32(adapter->mmio + 0x6890c, 1);

         reg_dvfs_cfg_en.reg.Dvfs_3d_Auto_En   = adapter->pm_caps.gfx_dvfs_auto;
         reg_dvfs_cfg_en.reg.Dvfs_Vcp0_Auto_En = adapter->pm_caps.vcp_dvfs_auto;
         reg_dvfs_cfg_en.reg.Dvfs_Vcp1_Auto_En = adapter->pm_caps.vcp_dvfs_auto;
         reg_dvfs_cfg_en.reg.Dvfs_Vpp_Auto_En  = adapter->pm_caps.vpp_dvfs_auto;
         if(adapter->pm_caps.gfx_mem_dvfs || adapter->pm_caps.vcp_mem_dvfs || adapter->pm_caps.vpp_mem_dvfs)
         {
            reg_dvfs_cfg_en.reg.Reg_Memory_Dvfs_En = 1;
         }

         zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Reg_Dvfs_Cfg_En_Offset * 4, reg_dvfs_cfg_en.uint);
     }
     else
     {
         vidsch_mgr_t  *sch_mgr = NULL;
         int engine_index;

         for (engine_index = adapter->active_engine_count-1; engine_index >= 0; engine_index--)
         {
              int ret = S_OK;
              unsigned long flags = 0;
              sch_mgr = adapter->sch_mgr[engine_index];

              if (sch_mgr == NULL)  continue;
              vidsch_update_engine_idle_status(adapter, (1 << engine_index));
              // all active engines should be idle, otherwise will do nothing
              if (!sch_mgr->completely_idle)
              {
                  return;
              }
         }

         zx_write32(adapter->mmio + MMIO_CSP_START_ADDRESS + Reg_Reg_Dvfs_Cfg_En_Offset * 4, reg_dvfs_cfg_en.uint);
         // disable dvfs before turn off pll
         zx_write32(adapter->mmio + 0x68900, 0);
         zx_write32(adapter->mmio + 0x68904, 0);
         zx_write32(adapter->mmio + 0x68908, 0);
         zx_write32(adapter->mmio + 0x6890c, 0);

         // turn off epll
         epll_val = 0; // EPLL_PU | EPLL_RST
         zx_write32(adapter->mmio + 0x64018, epll_val);

         // turn off vpll
         vpll_val = 0; // VPLL_PU | VPLL_RST
         zx_write32(adapter->mmio + 0x64020, vpll_val);
     }

     schedule->pll_on = pll_on;
}

void vidsch_pll_tuning_e3k(adapter_t *adapter)
{
     
     vidsch_mgr_t  *sch_mgr = NULL;
     int engine_index;

     if (!adapter->pm_caps.pll_control ||
         !adapter->schedule->enable_timer)
     {
         return;
     }

     for (engine_index = adapter->active_engine_count-1; engine_index >= 0; engine_index--)
     {
          int ret = S_OK;
          unsigned long flags=0;
          sch_mgr = adapter->sch_mgr[engine_index];

          if (sch_mgr == NULL)  continue;
          vidsch_update_engine_idle_status(adapter, (1 << engine_index));
          // all active engines should be idle, otherwise will do nothing
          if (!sch_mgr->completely_idle)
          {
              return;
          }
     }

     zx_mod_timer(adapter->schedule->timer, adapter->timeout_ms);
}

