#include "zx_adapter.h"
#include "vidsch.h"
#include "vidschi.h"

/* must called sch_mgr->engine_lock locked */
void vidschi_engine_dvfs_power_on(vidsch_mgr_t *sch_mgr)
{
    adapter_t *adapter = sch_mgr->adapter;

    if (!adapter->pwm_level.EnableClockGating || !adapter->pm_caps.pwm_manual)
    {
        return;
    }

    if(!sch_mgr->engine_dvfs_power_on)
    {
        if(!list_empty(&sch_mgr->submitted_task_list) || sch_mgr->last_send_fence_id != sch_mgr->returned_fence_id)
        {
            if(sch_mgr->chip_func->power_clock)
            {
                sch_mgr->chip_func->power_clock(sch_mgr, FALSE);
                sch_mgr->engine_dvfs_power_on = TRUE;
            }
        }
    }
}

void vidschi_try_power_tuning(adapter_t *adapter)
{
    vidschedule_t *schedule = adapter->schedule;

    if (!adapter->pwm_level.EnableClockGating || !adapter->pm_caps.pwm_manual)
    {
        return;
    }

    if(schedule->chip_func->power_tuning)
    {
        schedule->chip_func->power_tuning(adapter);
    }
}

void vidschi_try_pll_tuning(adapter_t *adapter)
{
     vidschedule_t *schedule = adapter->schedule;

     if (!adapter->pm_caps.pll_control)
     {
         return;
     }

     if (schedule->chip_func->pll_tuning)
     {
         unsigned long flags = zx_spin_lock_irqsave(schedule->pll_lock);
         schedule->chip_func->pll_tuning(adapter);
         zx_spin_unlock_irqrestore(schedule->pll_lock, flags);
     }
}

void vidschi_timer_fn(void* data)
{
     vidschedule_t *schedule = (vidschedule_t*)data;
     adapter_t* adapter = schedule->adapter;

     if (adapter->pm_caps.pll_control && schedule->chip_func->pll_on_off)
     {
         zx_spin_lock(schedule->pll_lock);
         schedule->chip_func->pll_on_off(schedule->adapter, FALSE);
         zx_spin_unlock(schedule->pll_lock);
     }
}

