//*****************************************************************************
//  Copyright (c) 2018 Shanghai Zhaoxin Semiconductor Co., Ltd. 
//  All Rights Reserved.
//
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Shanghai Zhaoxin Semiconductor Co., Ltd.;
//  the contents of this file may not be disclosed to third parties, copied or
//  duplicated in any form, in whole or in part, without the prior written
//  permission of Shanghai Zhaoxin Semiconductor Co., Ltd.
//
//  The copyright of the source code is protected by the copyright laws of the People's
//  Republic of China and the related laws promulgated by the People's Republic of China
//  and the international covenant(s) ratified by the People's Republic of China. 
//*****************************************************************************
#include "zx_irq_e3k.h"

static void zx_translate_interrupt_bits(disp_info_t* disp_info, int sw2hw, intr_info_t* info, int* masks)
{
    if(!info || !masks)
    {
        return;
    }

    if(sw2hw)
    {
        info->biu_intr_bits = 0;
        info->csp_intr_bits = 0;

        info->biu_intr_bits |= (*masks & INT_VSYNC1)? VSYNC1_INT : 0;
        info->biu_intr_bits |= (*masks & INT_VSYNC2)? VSYNC2_INT : 0;
        info->biu_intr_bits |= (*masks & INT_DP1)? DP1_INT : 0;
        info->biu_intr_bits |= (*masks & INT_HDCODEC)? HDCODEC_INT : 0;

        if(disp_info->adp_info->chip_id == CHIP_ELT3K)
        {
            info->biu_intr_bits |= (*masks & INT_DP2)? DP2_INT : 0;
            info->biu_intr_bits |= (*masks & INT_DP3)? DP3_INT : 0;
            info->biu_intr_bits |= (*masks & INT_DP4)? DP4_INT : 0;
            info->biu_intr_bits |= (*masks & INT_VIP1)? VIP1_INT : 0;
            info->biu_intr_bits |= (*masks & INT_VIP2)? VIP2_INT : 0;
            info->biu_intr_bits |= (*masks & INT_VIP3)? VIP3_INT : 0;
            info->biu_intr_bits |= (*masks & INT_VIP4)? VIP4_INT : 0;
            info->biu_intr_bits |= (*masks & INT_VSYNC3)? VSYNC3_INT : 0;
            info->biu_intr_bits |= (*masks & INT_VSYNC4)? VSYNC4_INT : 0;

            info->csp_intr_bits |= (*masks & INT_FENCE)? ENGINE_FENCE_INT : 0;
            info->csp_intr_bits |= (*masks & INT_FE_HANG_VD0)? FE_HANG_VD0_INT : 0;
            info->csp_intr_bits |= (*masks & INT_BE_HANG_VD0)? BE_HANG_VD0_INT : 0;
            info->csp_intr_bits |= (*masks & INT_FE_ERROR_VD0)? FE_ERROR_VD0_INT : 0;
            info->csp_intr_bits |= (*masks & INT_BE_ERROR_VD0)? BE_ERROR_VD0_INT : 0;
            info->csp_intr_bits |= (*masks & INT_FE_HANG_VD1)? FE_HANG_VD1_INT : 0;
            info->csp_intr_bits |= (*masks & INT_BE_HANG_VD1)? BE_HANG_VD1_INT : 0;
            info->csp_intr_bits |= (*masks & INT_FE_ERROR_VD1)? FE_ERROR_VD1_INT : 0;
            info->csp_intr_bits |= (*masks & INT_BE_ERROR_VD1)? BE_ERROR_VD1_INT : 0;
        }
        else
        {
            info->biu_intr_bits |= (*masks & INT_VSYNC3)? VSYNC3_INT_CHX004 : 0;
            info->biu_intr_bits |= (*masks & INT_DP2)? DP2_INT_CHX004 : 0;

            info->csp_intr_bits |= (*masks & INT_FENCE)? ENGINE_FENCE_INT_CHX004 : 0;   

            info->csp_intr_bits |= (*masks & INT_FE_HANG_VD0)? FE_HANG_VD0_INT : 0;
            info->csp_intr_bits |= (*masks & INT_BE_HANG_VD0)? BE_HANG_VD0_INT : 0;
            info->csp_intr_bits |= (*masks & INT_FE_ERROR_VD0)? FE_ERROR_VD0_INT : 0;
            info->csp_intr_bits |= (*masks & INT_BE_ERROR_VD0)? BE_ERROR_VD0_INT : 0;
            info->csp_intr_bits |= (*masks & INT_FE_HANG_VD1)? FE_HANG_VD1_INT : 0;
            info->csp_intr_bits |= (*masks & INT_BE_HANG_VD1)? BE_HANG_VD1_INT : 0;
            info->csp_intr_bits |= (*masks & INT_FE_ERROR_VD1)? FE_ERROR_VD1_INT : 0;
            info->csp_intr_bits |= (*masks & INT_BE_ERROR_VD1)? BE_ERROR_VD1_INT : 0;

            info->csp_intr_bits |= (*masks & INT_DVFS_3D_CHECKSIZE)? DVFS_3D_CHECKSIZE_INT : 0;   
            info->csp_intr_bits |= (*masks & INT_DVFS_VCP0_CHECKSIZE)? DVFS_VCP0_CHECKSIZE_INT : 0;   
            info->csp_intr_bits |= (*masks & INT_DVFS_VCP1_CHECKSIZE)? DVFS_VCP1_CHECKSIZE_INT : 0;   
            info->csp_intr_bits |= (*masks & INT_DVFS_VPP_CHECKSIZE)? DVFS_VPP_CHECKSIZE_INT : 0;   

            info->csp_intr_bits |= (*masks & INT_DVFS_3D_CHANGE)? DVFS_3D_CHANGE_INT : 0;   
            info->csp_intr_bits |= (*masks & INT_DVFS_VCP0_CHANGE)? DVFS_VCP0_CHANGE_INT : 0;   
            info->csp_intr_bits |= (*masks & INT_DVFS_VCP1_CHANGE)? DVFS_VCP1_CHANGE_INT : 0;   
            info->csp_intr_bits |= (*masks & INT_DVFS_VPP_CHANGE)? DVFS_VPP_CHANGE_INT : 0;   
        }
    }
    else
    {
        *masks = 0;
        *masks |= (info->biu_intr_bits & VSYNC1_INT)? INT_VSYNC1 : 0;
        *masks |= (info->biu_intr_bits & VSYNC2_INT)? INT_VSYNC2 : 0;
        *masks |= (info->biu_intr_bits & DP1_INT)? INT_DP1 : 0;
        *masks |= (info->biu_intr_bits & HDCODEC_INT)? INT_HDCODEC : 0;

        if(disp_info->adp_info->chip_id == CHIP_ELT3K)
        {
            *masks |= (info->biu_intr_bits & VSYNC3_INT)? INT_VSYNC3 : 0;
            *masks |= (info->biu_intr_bits & VSYNC4_INT)? INT_VSYNC4 : 0;
            *masks |= (info->biu_intr_bits & DP2_INT)? INT_DP2 : 0; 
            *masks |= (info->biu_intr_bits & DP3_INT)? INT_DP3 : 0;
            *masks |= (info->biu_intr_bits & DP4_INT)? INT_DP4 : 0;
            *masks |= (info->biu_intr_bits & VIP1_INT)? INT_VIP1 : 0;
            *masks |= (info->biu_intr_bits & VIP2_INT)? INT_VIP2 : 0;
            *masks |= (info->biu_intr_bits & VIP3_INT)? INT_VIP3 : 0;
            *masks |= (info->biu_intr_bits & VIP4_INT)? INT_VIP4 : 0;

            *masks |= (info->csp_intr_bits & ENGINE_FENCE_INT)? INT_FENCE : 0;
            *masks |= (info->csp_intr_bits & FE_HANG_VD0_INT)? INT_FE_HANG_VD0 : 0;
            *masks |= (info->csp_intr_bits & BE_HANG_VD0_INT)? INT_BE_HANG_VD0 : 0;
            *masks |= (info->csp_intr_bits & FE_ERROR_VD0_INT)? INT_FE_ERROR_VD0 : 0;
            *masks |= (info->csp_intr_bits & BE_ERROR_VD0_INT)? INT_BE_ERROR_VD0 : 0;
            *masks |= (info->csp_intr_bits & FE_HANG_VD1_INT)? INT_FE_HANG_VD1 : 0;
            *masks |= (info->csp_intr_bits & BE_HANG_VD1_INT)? INT_BE_HANG_VD1 : 0;
            *masks |= (info->csp_intr_bits & FE_ERROR_VD1_INT)? INT_FE_ERROR_VD1 : 0;
            *masks |= (info->csp_intr_bits & BE_ERROR_VD1_INT)? INT_BE_ERROR_VD1 : 0;
        }
        else
        {
            *masks |= (info->biu_intr_bits & VSYNC3_INT_CHX004)? INT_VSYNC3 : 0;
            *masks |= (info->biu_intr_bits & DP2_INT_CHX004)? INT_DP2 : 0; 
            
            *masks |= (info->csp_intr_bits & ENGINE_FENCE_INT_CHX004)? INT_FENCE : 0;
            *masks |= (info->csp_intr_bits & FE_HANG_VD0_INT)? INT_FE_HANG_VD0 : 0;
            *masks |= (info->csp_intr_bits & BE_HANG_VD0_INT)? INT_BE_HANG_VD0 : 0;
            *masks |= (info->csp_intr_bits & FE_ERROR_VD0_INT)? INT_FE_ERROR_VD0 : 0;
            *masks |= (info->csp_intr_bits & BE_ERROR_VD0_INT)? INT_BE_ERROR_VD0 : 0;
            *masks |= (info->csp_intr_bits & FE_HANG_VD1_INT)? INT_FE_HANG_VD1 : 0;
            *masks |= (info->csp_intr_bits & BE_HANG_VD1_INT)? INT_BE_HANG_VD1 : 0;
            *masks |= (info->csp_intr_bits & FE_ERROR_VD1_INT)? INT_FE_ERROR_VD1 : 0;
            *masks |= (info->csp_intr_bits & BE_ERROR_VD1_INT)? INT_BE_ERROR_VD1 : 0;

            *masks |= (info->csp_intr_bits & DVFS_3D_CHECKSIZE_INT)? INT_DVFS_3D_CHECKSIZE : 0;   
            *masks |= (info->csp_intr_bits & DVFS_VCP0_CHECKSIZE_INT)? INT_DVFS_VCP0_CHECKSIZE : 0;   
            *masks |= (info->csp_intr_bits & DVFS_VCP1_CHECKSIZE_INT)? INT_DVFS_VCP1_CHECKSIZE : 0;   
            *masks |= (info->csp_intr_bits & DVFS_VPP_CHECKSIZE_INT)? INT_DVFS_VPP_CHECKSIZE : 0;   

            *masks |= (info->csp_intr_bits & DVFS_3D_CHANGE_INT)? INT_DVFS_3D_CHANGE : 0;   
            *masks |= (info->csp_intr_bits & DVFS_VCP0_CHANGE_INT)? INT_DVFS_VCP0_CHANGE : 0;   
            *masks |= (info->csp_intr_bits & DVFS_VCP1_CHANGE_INT)? INT_DVFS_VCP1_CHANGE : 0;   
            *masks |= (info->csp_intr_bits & DVFS_VPP_CHANGE_INT)? INT_DVFS_VPP_CHANGE : 0;   

        }
    }
}

static int zx_get_default_intr_e3k(disp_info_t* disp_info)
{
    int intr = DEF_INTR;
    adapter_info_t *adapter = disp_info->adp_info;
    zx_card_t      *zx_card = disp_info->zx_card;
    krnl_adapter_init_info_t *info  = &zx_card->a_info;

    if(disp_info->adp_info->chip_id == CHIP_CHX004)
    {
        //if(info->zx_dvfs_mode == 0)
        {
            intr &= ~INT_DVFS;
        }
    }
    return intr;
}

static int zx_get_intr_enable_mask_e3k(disp_info_t* disp_info)
{
    adapter_info_t*  adapter = disp_info->adp_info;
    intr_info_t intr_info = {0};
    int masks = 0;
  
    intr_info.biu_intr_bits = zx_read32(adapter->mmio + INTR_EN_REG);
    intr_info.csp_intr_bits = zx_read32(adapter->mmio + ADV_INTR_EN_REG);

    zx_translate_interrupt_bits(disp_info, 0, &intr_info, &masks);

    return masks;    
}

static void zx_set_intr_enable_mask_e3k(disp_info_t* disp_info, int masks)
{
    adapter_info_t*  adapter = disp_info->adp_info;
    intr_info_t intr_info = {0};
    unsigned char mm8aa0;
    
    zx_translate_interrupt_bits(disp_info, 1, &intr_info, &masks);

    zx_write32(adapter->mmio + INTR_EN_REG, intr_info.biu_intr_bits);

    zx_write32(adapter->mmio + ADV_INTR_EN_REG, intr_info.csp_intr_bits);

    mm8aa0 = zx_read8(adapter->mmio + 0x8AA0);
    if(!masks && (mm8aa0 & 0x01))
    {
        //disable chip interrupt case
        zx_write8(adapter->mmio + 0x8AA0, mm8aa0 & 0xFE);    //disable adapter interrupt
        zx_write32(adapter->mmio + INTR_SHADOW_REG, 0x00);   //Clear all interrupts with any write to MM8574
    }
    else if(masks && !(mm8aa0 & 0x01))
    {
        //enable chip interrupt case
        mm8aa0 = zx_read8(adapter->mmio + 0x8AA0);
        zx_write8(adapter->mmio + 0x8AA0, mm8aa0 | 0x01);
        zx_write32(adapter->mmio + INTR_SHADOW_REG, 0x00);   //Clear all interrupts with any write to MM8574
    }
}

static void zx_enable_msi_e3k(disp_info_t* disp_info)
{
    unsigned int        tmp;
    adapter_info_t*  adapter = disp_info->adp_info;
    int  msi_reg1, msi_reg2;

    if(disp_info->adp_info->chip_id == CHIP_ELT3K)
    {
        msi_reg1 = 0x804C;
        msi_reg2 = 0x8048;
    }
    else
    {
        msi_reg1 = 0x80C4;
        msi_reg2 = 0x80C0;
    }

    //when MSI is turned on from registry,the MSI address should have be written here by OS

    tmp = zx_read32(adapter->mmio + msi_reg1);

    if (tmp != 0)
    {
        tmp = zx_read32(adapter->mmio + msi_reg2);
        if ((tmp & 0x10000) == 0)
        {
            // Before turn on MSI we need clear all pending interrupts, clear all BIU interrupts
            zx_write32(adapter->mmio + INTR_SHADOW_REG, 0);

            zx_write32(adapter->mmio + msi_reg2, tmp | 0x10000);
        }
    }
}

static void zx_disable_msi_e3k(disp_info_t* disp_info)
{
    unsigned int        temp;
    adapter_info_t*  adapter = disp_info->adp_info;
    int msi_reg;

    if(disp_info->adp_info->chip_id == CHIP_ELT3K)
    {
        msi_reg = 0x8048;
    }
    else
    {
        msi_reg = 0x80C0;
    }

    temp = zx_read32(adapter->mmio + msi_reg);
    temp &= ~0x10000;
    
    zx_write32(adapter->mmio + msi_reg, temp);
}

static int zx_get_interrupt_mask_e3k(disp_info_t* disp_info)
{
    adapter_info_t*  adapter = disp_info->adp_info;
    intr_info_t intr_info = {0};
    int intr_masks = 0;

    zx_write32(adapter->mmio + INTR_SHADOW_REG, 0);
    zx_mb();

    intr_info.biu_intr_bits = zx_read32(adapter->mmio + INTR_SHADOW_REG);
    intr_info.csp_intr_bits = zx_read32(adapter->mmio + ADV_SHADOW_REG);
    zx_translate_interrupt_bits(disp_info, 0, &intr_info, &intr_masks);

    return intr_masks;
}

irq_chip_funcs_t irq_chip_funcs = 
{
    .get_default_intr = zx_get_default_intr_e3k,
    .get_intr_enable_mask = zx_get_intr_enable_mask_e3k,
    .set_intr_enable_mask = zx_set_intr_enable_mask_e3k,
    .enable_msi = zx_enable_msi_e3k,
    .disable_msi = zx_disable_msi_e3k,
    .get_interrupt_mask = zx_get_interrupt_mask_e3k,
};
