//*****************************************************************************
//  Copyright (c) 2018 Shanghai Zhaoxin Semiconductor Co., Ltd. 
//  All Rights Reserved.
//
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Shanghai Zhaoxin Semiconductor Co., Ltd.;
//  the contents of this file may not be disclosed to third parties, copied or
//  duplicated in any form, in whole or in part, without the prior written
//  permission of Shanghai Zhaoxin Semiconductor Co., Ltd.
//
//  The copyright of the source code is protected by the copyright laws of the People's
//  Republic of China and the related laws promulgated by the People's Republic of China
//  and the international covenant(s) ratified by the People's Republic of China. 
//***************************************************************************** 

/*
* ZX SDK TYPES
*/
#ifndef __ZX_TYPES_H__
#define __ZX_TYPES_H__

#include "zx_def.h"

typedef enum
{
    ZX_USAGE_DISPLAY_SURFACE   = 0x00000001,
    ZX_USAGE_OVERLAY_SURFACE   = 0x00000002,
    ZX_USAGE_CURSOR_SURFACE    = 0x00000004,
    ZX_USAGE_RENDER_TARGET     = 0x00000008,
    ZX_USAGE_TEXTURE_BUFFER    = 0x00000010,
    ZX_USAGE_SHADER_BUFFER     = 0x00000020,
    ZX_USAGE_DATA_BUFFER       = 0x00000040,
    ZX_USAGE_TEMP_BUFFER       = 0x00000080,
    ZX_USAGE_COMPOSER_SURFACE  = 0x00000100,
    ZX_USAGE_SHADOW_SURFACE    = 0x00000200,
    ZX_USAGE_VIDEO             = 0x00000400,
    ZX_USAGE_VIDEO_DECODER     = 0x00000400,
    ZX_USAGE_CAMERA            = 0x00000800,
    ZX_USAGE_VIDEO_ENCODER     = 0x00001000,
    ZX_USAGE_RENDER_SCRIPT     = 0x00002000,

    ZX_USAGE_CREATE_PIXMAP     = 0x01000000,
    ZX_USAGE_MIU_COUNTER       = 0x02000000,

    ZX_USAGE_FORCE_PCIE        = 0x00004000,
    ZX_USAGE_FORCE_LOCAL       = 0x00008000,
    ZX_USAGE_HWCTX_BUFFER      = 0x00010000,
    ZX_USAGE_FRAMEBUFFER       = 0x08000000,
    ZX_USAGE_TRY_TILED         = 0x80000000,
}zx_usage_bit;

typedef enum
{
    ZX_ACCESS_UNKNOWN    = 0,
    ZX_ACCESS_GPU_ONLY   = 1,
    ZX_ACCESS_GPU_ALMOST,
    ZX_ACCESS_CPU_ALMOST,
//    ZX_ACCESS_CPU_ONLY,
}zx_access_hint;

/* ZX Format description:
    All the format define as big endian byte order
    the first named component is in the least-significant bits
    This format  as same as HSF_XXX
    For example:
    ZX_FORMAT_R8G8B8A8_UNORM 
        char *pR8G8B8A8 = 
        pR8G8B8A8[0] = r;  //R
        pR8G8B8A8[1] = g; //G
        pR8G8B8A8[2] = b; //B
        pR8G8B8A8[3] = a; //A

    ZX_FORMAT_B5G6R5_UNORM
    unsigned short * p565Color
        *p565Color = b | r << 5 | g << 11;
        
*/
typedef enum
{
    ZX_FORMAT_A8_UNORM       = 1,
    ZX_FORMAT_B5G6R5_UNORM   = 2,        /*rmask 0xf800; gmask 0x7e0; bmask 0x1f*/
    ZX_FORMAT_B5G5R5A1_UNORM = 3,
    ZX_FORMAT_A1B5G5R5_UNORM = 4,
    ZX_FORMAT_B4G4R4A4_UNORM = 5,
    ZX_FORMAT_A4B4G4R4_UNORM = 6,
    ZX_FORMAT_B8G8R8A8_UNORM = 7,      /*rmask 0x00ff0000; gmask 0x0000ff00; bmask 0x000000ff; amask 0xff000000*/
    ZX_FORMAT_B8G8R8X8_UNORM = 8,
    ZX_FORMAT_R8G8B8A8_UNORM = 9,      /*rmask 0x000000ff; gmask 0x0000ff00; bmask 0x00ff0000; amask 0xff000000*/
    ZX_FORMAT_R8G8B8X8_UNORM = 10,
    ZX_FORMAT_A8R8G8B8_UNORM = 11,
    ZX_FORMAT_YUY2           = 12,
    ZX_FORMAT_NV12_LINEAR    = 13,
    ZX_FORMAT_NV12_TILED     = 14,
    ZX_FORMAT_NV21_LINEAR    = 15,
    ZX_FORMAT_YV12           = 16,
    ZX_FORMAT_FLOAT32        = 17,
    ZX_FORMAT_UINT32         = 18,
    ZX_FORMAT_INT32          = 19,
    ZX_FORMAT_R8_UNORM       = 20,

    ZX_FORMAT_RAW10          = 21,
    ZX_FORMAT_RAW16          = 22,
    ZX_FORMAT_RAW_OPAQUE     = 23,
    ZX_FORMAT_B10G10R10A2_UNORM = 24,
    ZX_FORMAT_R8G8_UNORM     = 25,
    ZX_FORMAT_R16_UNORM     = 26,
    ZX_FORMAT_R16G16_UNORM     = 27,
    ZX_FORMAT_P010             = 28,

    ZX_FORMAT_B8G8R8A8_SRGB  = 29,
    ZX_FORMAT_B8G8R8X8_SRGB  = 30,
}zx_format;

typedef struct
{
    unsigned int device;            /* in */
    unsigned int reference;         /* in, for rename create, if set this field, ignore all follow input field */
    unsigned int width;             /* in */
    unsigned int height;            /* in */
    unsigned int usage_mask;        /* in */
    zx_format   format;            /* in */
    zx_ptr64_t   user_ptr;         /* in */
    unsigned int user_buf_size;     /* in */
    zx_access_hint access_hint;    /* in */
    unsigned int fence_sync  : 1;   /* in */
    unsigned int unpagable   : 1;   /* in/out */
    unsigned int tiled       : 1;   /* in/out */
    unsigned int secured     : 1;   /* in/out */
    unsigned int compressed  : 1;   /* in/out */
    unsigned int primary     : 1;   /* in/out */
    unsigned int has_pages   : 1;   /* out */
    unsigned int force_clear : 1;   /* out */

    unsigned int hw_format;         /* out */
    unsigned int size;              /* out */
    unsigned int width_aligned;     /* out */
    unsigned int height_aligned;    /* out */
    unsigned int bit_cnt;           /* out */
    unsigned int pitch;             /* out */
    unsigned int tiled_width;       /* out */
    unsigned int tiled_height;      /* out */
    unsigned int allocation;        /* out */
    unsigned int sync_obj;          /* out */
    unsigned int compress_format;   /* out */
    unsigned int buffer_fd;         /* out */
    unsigned int segment_id;        /* out */
    unsigned long long gpu_virt_addr;     /* out */
    unsigned long long fence_addr;        /* out */
}zx_create_allocation_t;

typedef struct
{
    unsigned int device;            /* in */
    unsigned int allocation;        /* in */

    unsigned int size;              /* out */
    unsigned int alignment;         /* out */
    unsigned int width;             /* out */
    unsigned int height;            /* out */
    unsigned int aligned_width;     /* out */
    unsigned int aligned_height;    /* out */
    unsigned int tiled_width;       /* out */
    unsigned int tiled_height;      /* out */
    unsigned int bit_cnt;           /* out */
    unsigned int pitch;             /* out */
    unsigned int unpagable : 1;     /* out */
    unsigned int tiled     : 1;     /* out */
    unsigned int secured   : 1;     /* out */
    unsigned int snoop     : 1;     /* out */
    unsigned int local     : 1;     /* out */
    unsigned int ForceLocal: 1;     /* out */
    unsigned int has_pages : 1;     /* out */
    unsigned int cpu_visible    :1; /* out */
    unsigned int force_clear : 1;   /* out */

    unsigned int compress_format;   /* out */
    unsigned int hw_format;         /* out */
    unsigned int segment_id;        /* out */
    unsigned int bl_slot_index;    /* out */
    unsigned int sync_obj;          /* out */

    unsigned long long gpu_virt_addr;     /* out */
    unsigned long long cpu_phy_addr;      /* out */
    unsigned long long fence_addr;        /* out */
}zx_open_allocation_t;

typedef struct
{
    unsigned int device;            /* in */
    unsigned int allocation;        /* in */
}zx_destroy_allocation_t;

typedef struct
{
    unsigned int device;         /* out */
}zx_create_device_t;

typedef struct
{
    unsigned int  device;                   /* in   */
    unsigned int  engine_index;             /* in   */
    unsigned int  flags;                    /* in   */
    unsigned int  context;                  /* out  */
}zx_create_context_t;

typedef struct
{
    unsigned int device;              /* in   */
    unsigned int context;             /* in   */
}zx_destroy_context_t;

typedef struct
{
    zx_ptr64_t     ptr;
    unsigned int    size;
    unsigned int    pad;
}zx_cmdbuf_t;

typedef struct
{
    unsigned int    ValidHead;
    unsigned int    HWCtxBufIndex;
    unsigned int    ZX_Interface_Version;
    unsigned int    Contain3dCmd                :1; // keep consistent with source_new, not used for elite
    unsigned int    ContainLpdpCmd              :1;
    unsigned int    CLCSOnly                    :1; // opencl without 2D/3D blt/L2 invalidate
    unsigned int    Flag2dCmd                   :1; // Xserver 2d command
    unsigned int    Flag3dbltCmd                :1; // Xserver composite 3d blt command
    unsigned int    DvfsForceLevel              :3;
    unsigned int    InitializeContext           :1;
    unsigned int    ContainDIP                  :1;
    unsigned int    TraceDMA                    :1;

    unsigned int    resize_command_buffer       :1; /* resize command buffer after render */
    unsigned int    resize_allocation_list      :1; /* resize allocation list after render */
    unsigned int    resize_patch_location_list  :1; /* resize patch location list after render */
    unsigned int    null_rendering              :1; /* null rendering, refer to WDK */
    unsigned int    dump_miu                    :1; /* indicate miu counter dump for context*/
} zx_render_flags_t;

typedef struct zx_allocation_list
{
    unsigned int       hAllocation;
    union
    {
        struct
        {
            unsigned int     WriteOperation       : 1;    // 0x00000001
            unsigned int     DoNotRetireInstance : 1;    // 0x00000002            
            unsigned int     Reserved             :30;    // 0xFFFFFFFC
        };
        unsigned int         Value;
    };
} zx_allocation_list_t;

typedef struct zx_patchlocation_list
{
    unsigned int AllocationIndex;
    union {
        struct
        {
            unsigned int SlotId          : 24;   // 0x00FFFFFF
            unsigned int Reserved        : 8;    // 0xFF000000
        };
        unsigned int     Value;
    };

    unsigned int     DriverId;
    unsigned int     AllocationOffset;
    unsigned int     PatchOffset;
    unsigned int     SplitOffset;
} zx_patchlocation_list_t;

typedef struct zx_syncobj_list
{
    unsigned int hSyncObject;
    unsigned int PatchOffset;
    unsigned long long FenceValue;
} zx_syncobj_list_t;

typedef struct
{
    unsigned int            context;                        /* in                   */
    unsigned int            command_length;                 /* in(number of bytes)  */
    unsigned int            allocation_count;               /* in                   */
    unsigned int            patch_location_count;           /* in                   */
    unsigned int            sync_object_count;              /* in                   */
    unsigned int            cmdbuf_count;                   /* in                   */
    zx_render_flags_t       flags;                          /* in                   */
    zx_ptr64_t              allocation_list;
    zx_ptr64_t              patch_location_list;
    zx_ptr64_t              sync_object_list;
    zx_ptr64_t              cmdbuf_array;
}zx_render_t;

/**************fence sync object relate***************/
#define ZX_SYNC_OBJ_TYPE_MUTEX            1
#define ZX_SYNC_OBJ_TYPE_SEMPAPHORE       2
#define ZX_SYNC_OBJ_TYPE_FENCE            3
#define ZX_SYNC_OBJ_TYPE_CPU_NOTIFICATION 4
#define ZX_SYNC_OBJ_TYPE_DMAFENCE         15
typedef struct
{
    unsigned int       type;
    unsigned int       device;                                /* in  */
    unsigned int       fence_sync_object;                     /* out */
    unsigned int       pad;
    union {
        struct {
            unsigned long long init_value;                    /* in  */
            unsigned long long fence_addr;                    /* out */
        } fence;

        struct {
            unsigned int        init_state;
        } mutex;

        struct {
            unsigned int        event;
        } cpu_notification;

        struct {
            unsigned int        max_count;
            unsigned int        init_count;
        } semaphore;
    };
}zx_create_fence_sync_object_t;

typedef struct
{
    unsigned int       device;                   /* in */
    unsigned int       fence_sync_object;        /* in */
}zx_destroy_fence_sync_object_t;

#define ZX_SYNC_OBJ_ERROR                -1
#define ZX_SYNC_OBJ_INVALID_ARGU         -2
#define ZX_SYNC_OBJ_ALREAD_SIGNALED       1
#define ZX_SYNC_OBJ_TIMEOUT_EXPIRED       2
#define ZX_SYNC_OBJ_CONDITION_SATISFIED   3
#define ZX_SYNC_OBJ_WAIT_ON_SERVER        4
#define ZX_SYNC_OBJ_UNSIGNALED            5

typedef struct
{
    unsigned int       context;                  /* in */
    unsigned int       fence_sync_object;        /* in */
    unsigned long long timeout;                  /* in nano_sec */  //timeout == 0, no wait just return current object status
    unsigned long long fence_value;              /* in */
    unsigned int       server_wait;              /* in */
    int                status;                   /* out */
}zx_wait_fence_sync_object_t;

#define ZX_SET_FENCE_VALUE   0x01
#define ZX_GET_FENCE_VALUE   0x02

typedef struct
{
    unsigned int       device;                   /* in */
    unsigned int       opcode;                   /* in */
    unsigned int       fence_sync_object;        /* in */
    int                appid;                    /* out */
    int                flip_limit;               /* out */
    int                discard_sync;             /* in & out*/
    unsigned long long fence_value;              /* out */
}zx_fence_value_t;

typedef struct
{
    unsigned int       context;                  /* in */
    unsigned int       server_wait;              /* in */
    unsigned long long timeout;                  /* in nano_sec */  //timeout == 0, no wait just return current object status
    int                status;                   /* out */
    int                pad;                   /* out */
}zx_wait_fence_t;

typedef struct zx_dvfs_clamp_status
{
    union
    {
        unsigned int uint;
        struct
        {
            unsigned int gfx_dvfs_index_max  : 4;
            unsigned int gfx_dvfs_index_min  : 4;
            unsigned int zxvd_dvfs_index_max : 4;
            unsigned int zxvd_dvfs_index_min : 4;
            unsigned int vpp_dvfs_index_max  : 4;
            unsigned int vpp_dvfs_index_min  : 4;
            unsigned int reserved            : 8;
        };
    };
}zx_dvfs_clamp_status_t;

typedef struct zx_dvfs_current_index
{
    union
    {
        int uint;
        struct
        {
            int gfx_index      : 4; 
            int zxvd_index     : 4;
            int vpp_index      : 4;
            int reserved       : 20;
        };
    };
}zx_dvfs_current_index_t;

typedef struct _zx_dvfs_set
{
    int engine_index;
    int table_index;
    int dvfs_force;
}zx_dvfs_set_t;

/*now only have perf mode, and only change dvfs clamp
slice switch will default use force two slice since power not saving muchand perf increase
*/

typedef enum
{
    ZX_POWER_MODE_PERF       = 0,   //force two slice, eclock can force
    ZX_POWER_MODE_SAVING     = 1,   //slice switch auto, eclock can force
    ZX_POWER_MODE_VIDEO      = 2,   //can use to force vcp/cpp 450M for 4k*2k video
    ZX_POWER_MODE_BALANCE    = 3,
    ZX_POWER_MODE_AGGRESSION = 4,
    ZX_POWER_MODE_NUM        = 5,
}zx_power_mode;

#define zx_align_down(val, align)  ((val) & ~((align) - 1))
#define zx_align_up(val, align)    (((val) + ((align) - 1)) & ~((align) - 1))

typedef enum
{
    PDISCARD  =  0,
    PLOW,
    PNORMAL,
    PHIGH,
    PMAX,
    PALL,
} zx_allocation_priority_t;


typedef struct
{
    unsigned int        Size;               // inout
    unsigned int        HwFormat;           // in
    unsigned int        HWCompressFormat;   // in
    unsigned int        AtType;             // in
    unsigned int        Priority;           // in
    unsigned int        Alignment;          // in
    unsigned int        PreferredSegment;   // in
    unsigned int        BitCount;           // in
    unsigned int        Width;              // in
    unsigned int        Height;             // in
    unsigned int        Pitch;              // in

    unsigned int        MaximumRenamingListLength;  // in

    unsigned int        bSecurity   :1;     // in
    unsigned int        CpuVisible  :1;     // in
    unsigned int        Primary     :1;     // in
    unsigned int        Swizzled    :1;     // in
    unsigned int        Video       :1;     // in
    unsigned int        Overlay     :1;     // in
    unsigned int        Camera      :1;     // in
    unsigned int        bNonSnoop   :1;     // in
    unsigned int        bVideoInternal  :1; // in
    unsigned int        Unpagable   :1;     // in
    unsigned int        FenceSync   :1;     // in, TODO(deleted)

    unsigned int        SetSegmentId : 1;   // in
    unsigned int        Cached : 1;         //in
    unsigned int        PoolType : 3;       //in
    unsigned int        CacheType : 2;      // in


    unsigned int        hAllocation;        // out
    unsigned int        FenceSyncObject;    // out
    unsigned int        pad;                // for 8byte alignment
    unsigned long long  FenceAddress;       // out

    unsigned long long  PhysAddr;           // out
    unsigned long long  Address;            // out
} zx_create_alloc_info_t;

typedef struct
{
    unsigned int        device;
    unsigned int        NumAllocations;
    zx_ptr64_t          pAllocationInfo;
} zx_create_resource_t;

typedef struct
{
    unsigned int handle;    // in
    unsigned int offset;    // in
    unsigned int size;      // in
    unsigned int readonly:1;// in
} zx_drm_gem_begin_cpu_access_t;

typedef struct
{
    unsigned int handle;
} zx_drm_gem_end_cpu_access_t;

typedef struct
{
    unsigned int gem_handle;
    unsigned int pad;
    unsigned long long offset;
    unsigned int prefault_num;
    unsigned int delay_map;
} zx_drm_gem_map_t;

typedef struct
{
    unsigned int crtc_id;   /* in */
    unsigned int pipe;      /* out */
} zx_kms_get_pipe_from_crtc_t;

typedef struct
{
    unsigned int    device;                /* in */
    unsigned int    context;               /* in */
    unsigned int    hw_ctx_buf_index;          /* in */
} zx_add_hw_ctx_buf_t;

typedef struct
{
    unsigned int    device;                /* in */
    unsigned int    context;               /* in */
    unsigned int    hw_ctx_buf_index;          /* in */
} zx_rm_hw_ctx_buf_t;

#define ZX_MISC_GET_3DBLT_CODE  (1)
typedef struct
{
    unsigned int size;          // output
    unsigned int capacity;      // output
    unsigned long long base;    // output
} zx_misc_get_3dblt_code_t;

#define ZX_MISC_SET_3DBLT_CODE  (2)
typedef struct
{
    unsigned int size;          // input
    unsigned int pad;
    zx_ptr64_t data;            // input
} zx_misc_set_3dblt_code_t;

#define ZX_MISC_MAP_GPUVA       (3)
typedef struct
{
    unsigned int allocation;    // input
    unsigned int pad;
    unsigned long long gpu_va;  // input
} zx_misc_map_gpuva_t;

#define ZX_MISC_VIDEO_FENCE_GET (4)
typedef struct
{
    unsigned int index;         // input
    unsigned int fence;         // output
} zx_misc_video_fence_get_t;

#define ZX_MISC_VIDEO_FENCE_CLEAR (5)
typedef struct
{
    unsigned int index; // input
} zx_misc_video_fence_clear_t;

typedef struct
{
    unsigned int op_code;
    unsigned int device;
    unsigned int context;
    unsigned int pad;
    union {
        zx_misc_get_3dblt_code_t    get_3dblt_code;
        zx_misc_set_3dblt_code_t    set_3dblt_code;
        zx_misc_map_gpuva_t         map_gpuva;
        zx_misc_video_fence_get_t   video_fence_get;
        zx_misc_video_fence_clear_t video_fence_clear;
    };
} zx_cil2_misc_t;


typedef struct
{
    unsigned long long video_mem_start;
    unsigned long long local_video_mem_length;
    unsigned long long pcie_video_mem_length;  //suppose pcie and local are continous in virtual address
} zx_delay_init_card_t;

#define ZX_DEBUGFS_GEM_ENABLE  0x1
#endif /* __ZX_TYPES_H__*/

